<?php

class WooSocialBackendFestiPlugin extends WooSocialFestiPlugin
{
    protected $_menuOptions = array(
        'general' => "General",
        'facebook' => "Facebook",
        'vk' => "VK",
        'gplus' => 'Google',
        'twitter' => 'Twitter',
        'paypal' => 'PayPal'
    );

    protected $_defaultMenuOption = 'general';
    
    protected function onInit()
    {
        $this->onInitSocialOpenGraph();
        $this->addActionListener('admin_menu', 'onAdminMenuAction', 100);
  
        $this->addActionListener('init', 'onRewriteRulesAction');
    } // end onInit
    
    public function onInitSocialOpenGraph()
    {
        $path = $this->getPluginPath();
        
        $path .= 'common/social/OpenGraphBackendWordpressAdapter.php';

        require_once $path;
        
        $openGraph = new OpenGraphBackendWordpressAdapter($this);
    } // end onInitSocialOpenGraph

    public function onInstall($refresh = false, $settings = false)
    {        
        if (!$this->_fileSystem) {
            $this->_fileSystem = $this->getFileSystemInstance();
        }
        
        if ($this->_hasPermissionToCreateCacheFolder()) {
            $this->_fileSystem->mkdir($this->_pluginCachePath, 0777);
        }
        
        if (!$refresh) {
            $settings = $this->getOptions('settings');    
        }
              
        if (!$refresh && !$settings) {
            $this->_doInitDefaultOptions('general');
            $this->_doInitDefaultOptions('facebook');
            $this->_doInitDefaultOptions('vk');
            $this->_doInitDefaultOptions('gplus');
            $this->_doInitDefaultOptions('twitter');
            $this->_doInitDefaultOptions('paypal');
        }
            
        $this->onRewriteRulesAction();
        
        flush_rewrite_rules();
    } // end onInstal
    
    public function onRewriteRulesAction()
    {
        parent::onRewriteRulesAction();
        $this->_onAppendPayPallRewriteRule();
    } // end onRewriteRulesAction

    private function _onAppendPayPallRewriteRule()
    {
        add_rewrite_rule(
            '^festi-api/auth/(paypal)/([^/]*)/?',
            'index.php?social[name]=$matches[1]&social[action]=$matches[2]',
            'top'
        );
    } // end _onAppendPayPallRewriteRule
    
    private function _doInitDefaultOptions($option)
    {
        $methodName = 'loadSettings';

        $method = array($this, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }

        $options = call_user_func_array($method, array($option));
        foreach ($options as $ident => &$item) {
            if ($this->_hasDefaultValueInItem($item)) {
                $values[$ident] = $item['default'];
            }
        }
        unset($item);
        
        $this->updateOptions($option, $values);
    } // end _doInitDefaultOptions
    
    private function _hasDefaultValueInItem($item)
    {
        return isset($item['default']);
    } //end _hasDefaultValueInItem
    
    private function _hasPermissionToCreateCacheFolder()
    {
        return ($this->_fileSystem->is_writable($this->_pluginPath)
               && !file_exists($this->_pluginCachePath));
    } // end _hasPermissionToCreateFolder
    
    public function getPluginTemplatePath($fileName)
    {
        return $this->_pluginTemplatePath.'backend/'.$fileName;
    } // end getPluginTemplatePath
    
    public function getPluginCssUrl($fileName) 
    {
        return $this->_pluginCssUrl.'backend/'.$fileName;
    } // end getPluginCssUrl
    
    public function getPluginJsUrl($fileName)
    {
        return $this->_pluginJsUrl.'backend/'.$fileName;
    } // end getPluginJsUrl
    
    public function _onFileSystemInstanceAction()
    {
        $this->_fileSystem = $this->getFileSystemInstance();
    } // end _onFileSystemInstanceAction
    
    public function onAdminMenuAction() 
    {
        $page = add_menu_page(
            __('WooSocial Settings', $this->_languageDomain), 
            __('WooSocial', $this->_languageDomain), 
            'manage_options', 
            'festi-social-login', 
            array(&$this, 'onDisplayOptionPage'), 
            $this->getPluginImagesUrl('icon_16x16.png')
        );
        
        $this->addActionListener(
            'admin_print_styles-'.$page, 
            'onInitCssAction'
        );
        
        $this->addActionListener(
            'admin_print_scripts-'.$page, 
            'onInitJsAction'
        );
        
        $this->addActionListener(
            'admin_head-'.$page,
            '_onFileSystemInstanceAction'
        );
    } // end onAdminMenuAction
    
    protected function hasOptionPageInRequest()
    {
        return array_key_exists('tab', $_GET)
               && array_key_exists($_GET['tab'], $this->_menuOptions);
    } // end hasOptionPageInRequest
    
    public function onInitJsAction()
    {
        $this->onEnqueueJsFileAction('jquery');
        $this->onEnqueueJsFileAction(
            'festi-user-role-prices-general',
            'general.js',
            'jquery',
            $this->version
        );
    } // end onInitJsAction
    
    public function onInitCssAction()
    {
        $this->onEnqueueCssFileAction(
            'festi-user-role-prices-styles',
            'style.css',
            array(),
            $this->version
        );
        
        $this->onEnqueueCssFileAction(
            'festi-admin-menu',
            'menu.css',
            array(),
            $this->version
        );
    } // end onInitCssAction
    
    public function onDisplayOptionPage()
    {
        if ($this->_isRefreshPlugin()) {
            $this->onRefreshPlugin();
        }
        
        if ($this->_isRefreshCompleted()) {
            $message = __(
                'Success refresh plugin',
                $this->_languageDomain
            );
            
            $this->displayUpdate($message);   
        }
        
        $this->_displayPluginErrors();
        
        
        if ($this->_menuOptions) {         
            $menu = $this->fetch('menu.phtml');
            echo $menu;
        }
        
        $methodName = 'fetchOptionPage';
        
        if ($this->hasOptionPageInRequest()) {
            $postfix = $_GET['tab'];
        } else {
            $postfix = $this->_defaultMenuOption;
        }
        $methodName.= ucfirst($postfix);
        
        $method = array(&$this, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }
        
        call_user_func_array($method, array());
    } // end onDisplayOptionPage
    
    public function displayOptionPage($optionName)
    {
         $vars = array(
            'optionName' => $optionName
        );

       // $this->displayOptionsHeader();
        if ($this->isUpdateOptions()) {
            try {
                $this->_doUpdateOptions($_POST['__action'], $_POST);
                           
                $this->displayOptionPageUpdateMessage(
                    'Success update settings'
                );               
            } catch (Exception $e) {
                $message = $e->getMessage();
                $this->displayError($message);
            }
        }

        $options = $this->getOptions($optionName);
        
        $vars['fieldset'] = $this->getOptionsFieldSet($optionName);        
        $vars['currentValues'] = $options;
        
        echo $this->fetch('settings_page.phtml', $vars);
    } // end displayOptionPage
    
    public function fetchOptionPageGeneral()
    {
        $optionName = 'general';
        
        $this->displayOptionPage($optionName);
    } // end fetchOptionPageSettings
    
    public function fetchOptionPageFacebook()
    {
        $optionName = 'facebook';

        $this->displayOptionPage($optionName);
    } // end fetchOptionPageSettings
    
    public function fetchOptionPageVk()
    {
        $optionName = 'vk';

        $this->displayOptionPage($optionName);
    } // end fetchOptionPageSettings
    
    public function fetchOptionPageGplus()
    {
        $optionName = 'gplus';

        $this->displayOptionPage($optionName);
    } // end fetchOptionPageGplus
    
    public function fetchOptionPageTwitter()
    {
        $optionName = 'twitter';

        $this->displayOptionPage($optionName);
    } // end fetchOptionPageTwitter
    
    public function fetchOptionPagePaypal()
    {
        $optionName = 'paypal';

        $this->displayOptionPage($optionName);
    } // end fetchOptionPagePaypal
    
    public function onRefreshPlugin()
    {
        $this->onInstall(true);
    } // end onRefreshPlugin
    
    private function _detectTheCacheFolderAccessErrors()
    {
        if (!$this->_fileSystem->is_writable($this->_pluginCachePath)) {

            $message = __(
                "Caching does not work! ",
                $this->_languageDomain
            );
            
            $message .= __(
                "You don't have permission to access: ",
                $this->_languageDomain
            );
            
            $path = $this->_pluginCachePath;
            
            if (!$this->_fileSystem->exists($path)) {
                $path = $this->_pluginPath;
            }
            
            $message .= $path;
            //$message .= $this->fetch('manual_url.phtml');
            
            $this->displayError($message);
            
            return true;
        }
        
        return false;
    } // end _detectTheCacheFolderAccessErrors
    
    
    public function getOptionsFieldSet($settingsName)
    {
        $methodName = 'getOptionsFieldSet';

        $methodName.= ucfirst($settingsName);
        
        $method = array(&$this, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }
        
        $fildset = call_user_func_array($method, array());

        $settings = $this->loadSettings($settingsName);

        if ($settings) {
            foreach ($settings as $ident => &$item) {
                if (array_key_exists('fieldsetKey', $item)) {
                   $key = $item['fieldsetKey'];
                   $fildset[$key]['filds'][$ident] = $settings[$ident];
                }
            }
            unset($item);
        }
        
        return $fildset;
    } // end getOptionsFieldSet
    
    
    public function getOptionsFieldSetGeneral()
    {
        $fildset = array(
            'general' => array(
            ),
        );
        
        return $fildset;
    } // end getOptionsFieldSet
    
    public function getOptionsFieldSetFacebook()
    {
        $fildset = array(
            'facebook' => array(
            ),
        );
        
        return $fildset;
    } // end getOptionsFieldSet
    
    public function getOptionsFieldSetVk()
    {
        $fildset = array(
            'vk' => array(
            ),
        );
        
        return $fildset;
    } // end getOptionsFieldSet
    
    public function getOptionsFieldSetGplus()
    {
        $fildset = array(
            'gplus' => array(
            ),
        );
        
        return $fildset;
    } // end getOptionsFieldSet
    
    public function getOptionsFieldSetTwitter()
    {
        $fildset = array(
            'twitter' => array(
            ),
        );
        
        return $fildset;
    } // end getOptionsTwitter
    
    public function getOptionsFieldSetPaypal()
    {
        $fildset = array(
            'paypal' => array(
            ),
        );
        
        return $fildset;
    } // end getOptionsPaypal

    public function loadSettings($settingsName)
    {
        $methodName = 'loadSettings';

        $methodName.= ucfirst($settingsName);
        
        $method = array(&$this, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }
        
        $settings = call_user_func_array($method, array());

        $values = $this->getOptions($settingsName);
        
        if ($values) {
            foreach ($settings as $ident => &$item) {
                if (array_key_exists($ident, $values)) {
                    $item['value'] = $values[$ident];
                }
            }
            unset($item);
        }
        
        return $settings;
    } // end loadSettings
    
    public function loadSettingsFacebook()
    {
        $settings = array(
            'appId' => array(
                'caption' => __(
                    'App ID',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'facebook',
                'default' => ''
            ),
            'appSecret' => array(
                'caption' => __(
                    'App Secret',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'facebook',
                'default' => ''
            ),
            
            'scope' => array(
                'caption' => __(
                    'Scope',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'facebook',
                'classes' => 'hide-option',
                'default' => 'public_profile,email'
            ),
            
            'loginButtonText' => array(
                'caption' => __(
                    'Login Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Log In With Facebook',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'facebook',
            ),
            'sharingButtonText' => array(
                'caption' => __(
                    'Share Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Share  to Facebook',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'facebook',
            ),
            'linkButtonText' => array(
                'caption' => __(
                    'Link Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Link your account to Facebook',
                    $this->_languageDomain
                ),
                //'fieldsetKey' => 'facebook',
            ),
        );
        return $settings;
    } // end loadSettings
    
    public function loadSettingsVk()
    {
        $settings = array(
            'appId' => array(
                'caption' => __(
                    'App ID',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'vk',
                'default' => ''
            ),
            'appSecret' => array(
                'caption' => __(
                    'App Secret',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'vk',
                'default' => ''
            ),
            
            'scope' => array(
                'caption' => __(
                    'Scope',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'vk',
                'default' => '4194367',
                'classes' => 'hide-option'
            ),
            
            'loginButtonText' => array(
                'caption' => __(
                    'Login Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Log In With VK',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'vk',
            ),
            'sharingButtonText' => array(
                'caption' => __(
                    'Share Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Share  to VK',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'vk',
            ),
            'linkButtonText' => array(
                'caption' => __(
                    'Link Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Link your account to VK',
                    $this->_languageDomain
                ),
                //'fieldsetKey' => 'vk',
            ),
        );
        return $settings;
    } // end loadSettings
    
    public function loadSettingsGplus()
    {
        $settings = array(
            'appId' => array(
                'caption' => __(
                    'App ID',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'gplus',
                'default' => ''
            ),
            'appSecret' => array(
                'caption' => __(
                    'App Secret',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'gplus',
                'default' => ''
            ),
            
            'scope' => array(
                'caption' => __(
                    'Scope',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'gplus',
                'default' => 'https://www.googleapis.com/auth/plus.login '.
                             'https://www.googleapis.com/auth/userinfo.email',
                'classes' => 'hide-option'
            ),
            
            'loginButtonText' => array(
                'caption' => __(
                    'Login Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Log In With Google',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'gplus',
            ),
            'sharingButtonText' => array(
                'caption' => __(
                    'Share Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Share  to Google',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'gplus',
            ),
            'linkButtonText' => array(
                'caption' => __(
                    'Link Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Link your account to Google',
                    $this->_languageDomain
                ),
               // 'fieldsetKey' => 'gplus',
            ),
        );
        return $settings;
    } // end loadSettings
    
    public function loadSettingsTwitter()
    {
        $settings = array(
            'appId' => array(
                'caption' => __(
                    'App ID',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'twitter',
                'default' => ''
            ),
            'appSecret' => array(
                'caption' => __(
                    'App Secret',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'twitter',
                'default' => ''
            ),
            
            'loginButtonText' => array(
                'caption' => __(
                    'Login Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Log In With Twitter',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'twitter',
            ),
            'sharingButtonText' => array(
                'caption' => __(
                    'Share Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Share  to Twitter',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'twitter',
            ),
            'linkButtonText' => array(
                'caption' => __(
                    'Link Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Link your account to Twitter',
                    $this->_languageDomain
                ),
            ),
        );
        return $settings;
    } // end loadSettings
    
    public function loadSettingsPaypal()
    {
        $settings = array(
            'appId' => array(
                'caption' => __(
                    'App ID',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'paypal',
                'default' => ''
            ),
            'appSecret' => array(
                'caption' => __(
                    'App Secret',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'paypal',
                'default' => ''
            ),
            'appRedirectUrl' => array(
                'caption' => __(
                    'Redirect Url',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'atributes' => array(
                    'readonly="readonly"'
                ),
                'class' => 'festi-social-paypal-redirect-url',
                'fieldsetKey' => 'paypal',
                'default' => $this->_getPaypalRedirectUrl(),
                'label' => __(
                    'Copy to PayPal App Redirect URLs',
                    $this->_languageDomain
                ),
            ),
            'scope' => array(
                'caption' => __(
                    'Scope',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'fieldsetKey' => 'paypal',
                'default' => 'openid profile email',
                'classes' => 'hide-option'
            ),
            
            'loginButtonText' => array(
                'caption' => __(
                    'Login Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Log In With PayPal',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'paypal',
            ),
            'sharingButtonText' => array(
                'caption' => __(
                    'Share Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Share  to PayPal',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'paypal',
            ),
            'linkButtonText' => array(
                'caption' => __(
                    'Link Button Text',
                    $this->_languageDomain
                ),
                'type' => 'input_text',
                'default' =>__(
                    'Link your account to PayPal',
                    $this->_languageDomain
                ),
               // 'fieldsetKey' => 'paypal',
            ),
        );
        return $settings;
    } // end loadPaypal
    
    private function _getPaypalRedirectUrl()
    {
        $url = get_site_url();
        
        $url .= '/festi-api/auth/paypal/login/';
        
        return $url;
    } // end _getPaypalRedirectUrl
  
    public function loadSettingsGeneral()
    {
        $settings = array(
            'enabledProviders' => array(
                'caption' => __(
                    'Enabled Social Provider',
                    $this->_languageDomain
                ),
                'type'    => 'multi_select',
                'fieldsetKey' => 'general',
                'values' => array(
                    'facebook' => __('Facebook', $this->_languageDomain),
                    'vk' => __('VK', $this->_languageDomain),
                    'gplus' => __('Google', $this->_languageDomain),
                    'twitter' => __('Twitter', $this->_languageDomain),
                    'paypal' => __('PayPal', $this->_languageDomain)
                ),
                'default' => array()
            ),
            'placeForDisplayLoginButtons' => array(
                'caption' => __(
                    'Display Social Login Button on',
                    $this->_languageDomain
                ),
                'type'    => 'multi_select',
                'fieldsetKey' => 'general',
                'values' => array(
                    'checkout' => __('Checkout', $this->_languageDomain),
                    'myaccount' => __('My Account', $this->_languageDomain),
                ),
                'default' => array()
            ),
            'textBeforeLoginButtons' => array(
                'caption' => __(
                    'Text Before the Login Buttons',
                    $this->_languageDomain
                ),
                'type' => 'textarea',
                'default' =>__(
                    'Save time next time you checkout by linking your account '.
                    'to your favorite social network. No need to remember '.
                    'another username and password.',
                    $this->_languageDomain
                ),
                'fieldsetKey' => 'general',
            ),
            'displayLinkAccountButtons' => array(
                'caption' => __(
                    'Display "Link Your Account" for registered users',
                    $this->_languageDomain
                ),
                'lable' => 'Enable "Link Your Account"',
                'type' => 'input_checkbox',
                //'fieldsetKey' => 'general'
            ),
            'textBeforeLinkAccountButtons' => array(
                'caption' => __(
                    'Text Before the "Link Your Account" Buttons',
                    $this->_languageDomain
                ),
                'type' => 'textarea',
                'default' =>__(
                    'Save time next time you checkout by linking your account '.
                    'to your favorite social network. No need to remember '.
                    'another username and password.',
                    $this->_languageDomain
                ),
                //'fieldsetKey' => 'general',
            ),
        );

        return $settings;
    } // end loadSettings
    
    private function _doUpdateOptions($optionName, $newSettings = array())
    {
        $this->updateOptions($optionName, $newSettings);
    } // end _doUpdateOptions
    
    public function displayOptionsHeader()
    { 
        $vars = array(
            'content' => __(
                'Woo Social Login Pro Options',
                $this->_languageDomain
            )
        );
        
        echo $this->fetch('options_header.phtml', $vars);
    } // end displayOptionsHeader
    
    private function _displayPluginErrors()
    {        
        $caheFolderErorr = $this->_detectTheCacheFolderAccessErrors();

        if ($caheFolderErorr) {
            echo $this->fetch('refresh.phtml');
        }
    } // end _displayPluginErrors
    
    public function isUpdateOptions()
    {
        return array_key_exists('__action', $_POST);
    } // end isUpdateOptions
    
    private function _isRefreshPlugin()
    {
        return array_key_exists('refresh_plugin', $_GET);
    } // end _isRefreshPlugin
    
    private function _isRefreshCompleted()
    {
        return array_key_exists('refresh_completed', $_GET);
    } // end _isRefreshCompleted
    
    public function displayOptionPageUpdateMessage($text)
    {
        $message = __(
            $text,
            $this->_languageDomain
        );
            
        $this->displayUpdate($message);   
    } // end displayOptionPageUpdateMessage
}