<?php

class WooSocialFrontendFestiPlugin extends WooSocialFestiPlugin
{
    protected $settings = array();

    private $_providers = array();
    
    protected function onInit()
    {
        $this->settings = $this->getOptions('general');
        
		$this->addActionListener('init', 'prepareProviders');

        $this->addActionListener('wp', 'onUserInitAction');
        $this->addActionListener('wp_head', 'onOpnenGraphInitAction');
        $this->addActionListener(
            'wp',
            'onInitHooksForDisplayAuthButtonsAction'
        );
        
        $this->addShortcodeListener(
            'woo_share',
            'onDisplayShareButtonsInShortcode'
        );
        
        $this->addShortcodeListener(
            'woo_social_login',
            'onDisplayLoginButtonsInShortcode'
        );
        
        $this->addActionListener('wp_print_styles', 'onInitCssAction');
        
        $this->addActionListener('wp_footer', 'onFooterAction');
        
        $this->addActionListener('wp_enqueue_scripts', 'onEnqueueScripts');
        
        $this->addActionListener(
            'wp_ajax_onAjaxSocialAction', 
            'onAjaxSocialAction'
        );
        
        $this->addActionListener(
            'wp_ajax_nopriv_onAjaxSocialAction', 
            'onAjaxSocialAction'
        );
    } // end onInit
	
    private function _hasSocailActionInQuery($query)
    {
        return array_key_exists('social', $query)
               && !empty($query['social'])
               && !array_key_exists('social', $_GET);
    } // end _hasSocailActionInQuery
	
    public function &getWpQueryInstance()
    {
        if (!$this->_hasWordpressQueryInGlobals()) {
            return false;
        }
        
        $wpQuery = $GLOBALS['wp_query'];

        return $wpQuery->query_vars;
    } // end getWoocommerceInstance
    
    private function _hasWordpressQueryInGlobals()
    {
        return array_key_exists('wp_query', $GLOBALS);
    } // end _hasWordpressQueryInGlobals
    
    public function onDisplayShareButtonsInShortcode($args = array())
    {
        if (!$this->_hasEnabledSocialProvidersInOptions()) {
            return '';
        }

        if ($args) {
            extract($args);
        }
        
        $type = (isset($type)) ? $type : false;
        
        $enabledButtons = $this->settings['enabledProviders'];

        $buttons = array();

        foreach ($enabledButtons as $button) {
            $buttons[] = $this->_providers[$button]->fetchShareButton($type);
        }
        
        if (!$buttons) {
            return '';
        }
        
        $vars = array(
            'buttons' => $buttons,
            'location' => 'horizontal'
        );
        
        if (isset($location)) {
            $vars['location'] = $location;
        }
        
        return $this->fetch('share_buttons_container.phtml', $vars);
    } // end onDisplayShareButtonsInShortcode
    
    public function onDisplayLoginButtonsInShortcode()
    {
        if (is_user_logged_in()) {
            return false;
        }
        
        return $this->fetchSocialLoginButtonsBlock();
    } // end onDisplayLoginButtonsInShortcode
    
    public function onOpnenGraphInitAction()
    {
        $path = $this->getPluginPath();
        
        $path .= 'common/social/OpenGraphFrontendWordpressAdapter.php';

        require_once $path;
        
        $openGraph = new OpenGraphFrontendWordpressAdapter($this);
        
        echo $openGraph->getMetaContainer();
    } // end onOpnenGraphInitAction

    public function onEnqueueScripts()
    {
        $version = $this->version;
        
        // XXX: Need for correct work ajax
        foreach ($this->_providers as $name => $provider) {
            $url = $this->getPluginStaticUrl().'js/frontend/'.$name.'.js';
            wp_enqueue_script($name, $url, array('jquery'), $version);
            
            wp_localize_script($name, 'AjaxData'.ucfirst($name), array(
                'ajaxurl'  => admin_url('admin-ajax.php'),
                'security' => wp_create_nonce($name),
                'provider' => $name
            ));
        }
    } // end onEnqueueScripts
    
    public function onAjaxSocialAction()
    {
        $result = array(
            'status' => false
        );
        if (!$this->_hasSocialProviderActionsInRequest($_POST)) {
            wp_send_json($result);
        }
        
        $socialAction = $this->_getSocialAction($_POST);
        $result['status'] = $this->_onSocialAction($socialAction, $_POST);
        
        wp_send_json($result);
    } // end onAjaxSocialAction
    
    public function prepareProviders()
    {
        if (empty($this->settings['enabledProviders'])) {
            return false;
        }
        
        foreach ($this->settings['enabledProviders'] as $providerName) {
            $this->_providers[$providerName] = $this->_createSocialAdapter(
                $providerName
            );
        }
    } // end prepareProviders
    
    
    private function _onSocialAction($data, $params = array())
    {
        if (!array_key_exists('name', $data) || empty($data['name'])) {
            throw new Exception("Undefined social name in request.");
        }
        
        if (!array_key_exists('action', $data) || empty($data['action'])) {
            $data['action'] = 'login';
        }
        
        if (!$this->_hasProvider($data['name'])) {
            return false;
        }
        
        $instance = $this->_providers[$data['name']];
        
        $data['action'] = filter_var($data['action'], FILTER_SANITIZE_STRING);
        $methodName = "on".ucfirst($data['action'])."Action";
        
        if ($data['action'] == 'login' && is_user_logged_in()) {
            return false;
        }
        
        $method = array(&$instance, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method ".$methodName);
        }
        
        return call_user_func_array($method, array($params));
    } // end _onSocialAction
    
    private function _hasProvider($name) {
        return array_key_exists($name, $this->_providers);
    } // end _hasProvider
    
    private function _createSocialAdapter($name)
    {
        $name = filter_var($name, FILTER_SANITIZE_STRING);
        
        $socialOptions = $this->getOptions($name);
        
        $className = ucfirst($name).'SocialComponent';
        $basePath = dirname(__FILE__).'/common/social/';
        $path = $basePath.$className.'.php';
        
        if (!file_exists($path)) {
            throw new Exception("Undefined class ".$className);
        }
        
        require_once $path;
        
        $social = new $className($socialOptions, $this);
        
        return new WordpressSocialAdapter($social, $this);
    } // end _getSocialAdapter

    public function getPluginTemplatePath($fileName)
    {
        return $this->_pluginTemplatePath."frontend/".$fileName;
    } // end getPluginTemplatePath
    
    public function getPluginCssUrl($fileName) 
    {
        return $this->_pluginCssUrl.'frontend/'.$fileName;
    } // end getPluginCssUrl
    
    public function onInitCssAction()
    {
        $this->onEnqueueCssFileAction(
            'festi-social-login-styles',
            'style.css',
            array(),
            $this->version
        );
        
        foreach ($this->_providers as $name => $provider) {
            $this->onEnqueueCssFileAction(
                'festi-social-login-'.$name.'-styles',
                $name.'.css',
                array(),
                $this->version
            );
        }
    } // end onInitCssAction

    public function onUserInitAction()
    {
		$query = $this->getWpQueryInstance();

        if ($this->_hasSocailActionInQuery($query)) {
             $socialKey = $query['social']['name']; 	
             $this->_providers[$socialKey]->doRedirectToAuthPage($query);
        }
		
        if (!$this->_hasSocialProviderActionsInRequest()) {
            return false;
        }
        
        $socialAction = $this->_getSocialAction();
        $this->_onSocialAction($socialAction);
    } // end onUserInitAction
    
    public function onInitHooksForDisplayAuthButtonsAction()
    {
        if (!$this->_hasEnabledSocialProvidersInOptions()
            || !$this->_hasDisplayLoginButtonsPlaceInOptions()) {
            return false;
        }        
            
        if (!is_user_logged_in()) {
            $this->addActionListener(
                'woocommerce_login_form_end',
                'appendSocialButtonsToLogInFormAction'
            );
            
            return true;
        }
    } // end onInitHooksForDisplayAuthButtonsAction
    
    private function _hasEnabledSocialProvidersInOptions($provider = false)
    {
        $result =  array_key_exists(
            'enabledProviders',
            $this->settings
        );
        
        if (!$result || !$provider) {
            return $result;
        }
        
        return in_array($provider, $this->settings['enabledProviders']);
    } // end _hasEnabledSocialProvidersInOptions
    
    private function _hasDisplayLoginButtonsPlaceInOptions()
    {
        return array_key_exists(
            'placeForDisplayLoginButtons',
            $this->settings
        );
    } // end _hasDisplayLoginButtonsPlaceInOptions
    
    public function appendSocialButtonsToLogInFormAction()
    {
        if (!$this->_isEnabledToDisplayInPage() || is_user_logged_in()) {
            return false;
        }
        
        echo $this->fetchSocialLoginButtonsBlock();
    } // appendSocialButtonsToLogInFormAction
    
    private function _isEnabledToDisplayInPage()
    {
        return $this->_isEnabledToDisplayInWoocommerceCheckoutPage()
               || $this->_isEnabledToDisplayInWoocommerceAccountPage();
    } // end _isEnabledToDisplayInPage
    
    private function _isEnabledToDisplayInWoocommerceCheckoutPage()
    {
        $option = $this->settings['placeForDisplayLoginButtons'];
        
        return is_checkout() && in_array('checkout', $option);
    } // end _isEnabledToDisplayInWoocommerceCheckoutPage
    
    private function _isEnabledToDisplayInWoocommerceAccountPage()
    {
        $option = $this->settings['placeForDisplayLoginButtons'];
        
        return is_account_page() && in_array('myaccount', $option);
    } // end _isEnabledToDisplayInWoocommerceAccountPage
    
    public function fetchSocialLoginButtonsBlock()
    {
        if (!$this->_hasEnabledSocialProvidersInOptions()) {
            return false;
        }   
        
        $enabledButtons = $this->settings['enabledProviders'];

        $buttons = array();

        foreach ($enabledButtons as $button) {
            $buttons[] = $this->_providers[$button]->fetchLoginButton();
        }
        
        $vars = array(
            'content' => $this->settings['textBeforeLoginButtons'],
            'buttons' => $buttons
        );
        
        return $this->fetch('social_buttons_container.phtml', $vars);
    } // end fetchSocialLinkButtonsBlock
    
    public function onFooterAction()
    {
        foreach ($this->_providers as $provider) {
            echo $provider->fetchFooter();
        }
    } // end onFooterAction
    
    private function _hasSocialProviderActionsInRequest($request = false)
    {
        if ($request === false) {
            $request = &$_GET;
        }
        
        return array_key_exists(
            WordpressSocialAdapter::SOCIAL_KEY_GET_REQUEST, 
            $request
        ) && !empty($request[WordpressSocialAdapter::SOCIAL_KEY_GET_REQUEST]);
    } // end _hasSocialProviderActionsInRequest
    
    private function _getSocialAction($request = false)
    {
        if ($request === false) {
            $request = &$_GET;
        }
        
        return $request[WordpressSocialAdapter::SOCIAL_KEY_GET_REQUEST];
    } // end _getSocialAction
    
}