<?php

abstract class AbstractSocialComponent implements ISocialComponent
{
    private $_options;
    
    public  $emailDomain = '';
	
	private $_engine;
    
    public function __construct($options, $engine)
    {
        $this->_options = $options;
		$this->_engine = $engine;
		
        $this->onInit();
    } // end __construct
    
    public function onInit()
    { 
    } // end onInit
    
    public function getOption($key)
    {
        if (!array_key_exists($key, $this->_options)) {
            return false;
        }
        
        return  $this->_options[$key];
    } // end getOption
    
    public function isValidateUser($email, $token)
    {
        $socialEmail = $this->getUserEmail($token);
        
        if (is_array($socialEmail)) {
            return in_array($email, $socialEmail);
        }
        
        return $socialEmail == $email;
    } // end isValidateUser
    
    public function getUserEmail($token)
    {
        $user = $this->getUserData($token);

        if (!$this->hasEmailInResponse($user)) {
            return $this->getEmailWithSocialId($user['id']);
        }

        return $user['email'];
    } // end getUserEmail
      
    protected function hasEmailInResponse($response)
    {
        return array_key_exists('email', $response);
    } // end _hasEmailInResponse
    
    public function getEmailWithSocialId($id)
    {
        return $id.'@'.$this->emailDomain;
    } // end getEmailWithSocialId
    
    public function doRedirectToAuthPage($query)
    {
        $url = $this->getAuthPageWithQuery($query);

        header("Location: ".$url);
        exit();
    } // end doRedirectToAuthPage
    
        
    public function getAuthPageUrl()
	{
		return $this->_engine->getAuthPageUrl();
	} // end getAuthPageUrl
	
	protected function getAuthPageWithQuery($query)
	{
		$url = $this->getAuthPageUrl();
		
		$args = array(
            'social' => $query['social']
        );

        $data = parse_url($_SERVER['REQUEST_URI']);
        
        if (array_key_exists('query', $data)) {
            parse_str($data['query'], $params);
            $args = array_merge($params, $args);
        }

        $seperator = preg_match("#\?#Umis", $url) ? '&' : '?';
        $url .= $seperator.http_build_query($args);

        return $url;
	} // end getAuthPageWithQuery
}