<?php
require_once dirname(__FILE__).'/../../libs/gplus/autoload.php';

class GplusSocialComponent extends AbstractSocialComponent
{
    private $_gplus;
    private $_info;
    public  $emailDomain = 'gmail.com';
    
    public function getName()
    {
        return 'gplus';
    }

    public function onInit()
    {
        $config = array(
          'appId'  => $this->getOption('appId'),
          'secret' => $this->getOption('appSecret'),
          'scope'  => $this->getOption('scope')
        );
        
        $this->_gplus = new Google_Client();
        $this->_gplus->setClientId($config['appId']);
        $this->_gplus->setClientSecret($config['secret']);
        $this->_gplus->setScopes(array($config['scope']));

        $this->_info = new Google_Service_Plus($this->_gplus);
    } // end onInit

    private function _setToken($token)
    {
        $this->_gplus->refreshToken($token);
    } // end _setToken
    
    public function getUserData($token)
    {
        $this->_setToken($token);
        $user = $this->_getUser();

        if (!$user) {
            $message = "The user is not logged in the".$this->getName();
            $message .= " social network.";
            throw new Exception($message);
        }

        $data = array(
            'id' => $user['id']
        );
        
        $emails = $this->_getEmailList($user);

        if ($emails) {
             $data['email'] = $emails;
        }

        return $data;
    } // end getUserData
    
    private function _getUser()
    {
       $user = $this->_info->people->get('me');
       return $user;
    } // end _getUser
    
    private function _getEmailList($user)
    {
        $emails = $user->getEmails();
        
        if (!$emails) {
            return false;
        }
        
        $emailList = array();
        
        foreach ($emails as $item) {
            if (!array_key_exists('value', $item)) {
                continue;
            }
            
            $emailList[] = $item['value'];
        }
        
        return $emailList;
    } // end _getEmailList
}