<?php

class OpenGraphBackendWordpressAdapter
{  
    protected $postTypes = array(
        'post',
        'product',
        'page'
    );
    private $_openGraphOptionName = 'festiSocialOpenGraph';
    private $_engine;

    public function __construct($engine)
    {
        $this->_engine = $engine;
        
        $this->onInit();
    } // __construct
    
    public function onInit()
    {
        $this->addActionListener(
            'add_meta_boxes',
            'onInitOpenGraphMetaAction'
        );
        
        $this->addActionListener(
            'admin_print_styles', 
            'onInitOpenGraphMetaCssAction'
        );
        
        $this->addActionListener(
            'save_post',
            'updateMetaOptionsAction',
            10,
            2
        );
    } // end onInit
    
    public function onInitOpenGraphMetaCssAction()
    {
        $this->_engine->onEnqueueCssFileAction(
            'festi-social-open-graph-meta',
            'open_graph_form.css',
            array(),
            $this->_engine->version
        );
    } // end onInitOpenGraphMetaCssAction
    
    public function onInitOpenGraphMetaAction() {
        foreach ($this->postTypes as $type) {
            add_meta_box(
                'festi-social-open-graph-meta',
                __('Social Open Graph', $this->_engine->_languageDomain), 
                array($this, 'onDisplayOpenGraphForm'),
                $type,
                'normal',
                'core'
            );
        }
    } // end onInitOpenGraphMetaAction
    
    public function onDisplayOpenGraphForm()
    {
        $vars = array(
            'options' => $this->getOpenGraphOptions(),
            'values' => $this->getOpenGraphOptionsValues(),
            'openGraphOptionName' => $this->_openGraphOptionName
        );
        
        echo $this->_engine->fetch('open_graph_meta_form.phtml', $vars);
    } // end onDisplayOpenGraphForm'
    
    public function getOpenGraphOptions()
    {
        $options = array(
            'title' => array(
                'caption' => __('Title', $this->_engine->_languageDomain),
                'type' => 'input_text'
            ),
            'description' => array(
                'caption' => __('Description', $this->_engine->_languageDomain),
                'type' => 'input_text'
            ),
            'image' => array(
                'caption' => __('Image Url', $this->_engine->_languageDomain),
                'type' => 'input_text'
            ),
        );
        
        return $options;
    } // end getOpenGraphOptions
    
    public function updateMetaOptionsAction($postId)
    {
        if (!$this->_hasMetaOptionsInRequest()) {
            return false;
        }

        $this->updateMetaOptions(
            $postId,
            $_POST[$this->_openGraphOptionName],
            $this->_openGraphOptionName
        );
    } // end updateMetaOptionsAction
    
    private function _hasMetaOptionsInRequest()
    {
        return array_key_exists($this->_openGraphOptionName, $_POST);
    } // end _hasMetaOptionsInRequest
    
    public function getOpenGraphOptionsValues()
    {
        $post = $this->getWordpressPostInstance();
        $postId = $post->ID;
        
        $options = $this->getMetaOptions($postId, $this->_openGraphOptionName);
        
        if (!$options) {
            $options = array();
        }
        
        return $options;
    } // end getOpenGraphOptionsValues
    
    public function &getWordpressPostInstance()
    {
        return $GLOBALS['post'];
    } // end getWoocommerceInstance
    
    public function updateMetaOptions($id, $value, $optionName)
    {
        $value = json_encode($value, JSON_UNESCAPED_UNICODE);
        
        update_post_meta(
            $id,
            $optionName,
            $value
        );
    } // end updateMetaOptions
    
    public function getMetaOptions($id, $optionName)
    {
        $value = get_post_meta($id, $optionName, true);
        
        if (!$value) {
            return false;
        }
        
        if (is_array($value)) {
            return $value;
        }
        
        $value = json_decode($value, true);
        
        return $value;
    } // end getMetaOptions
    
    public function addActionListener(
        $hook, $method, $priority = '', $args = ''
    )
    {
        add_action($hook, array(&$this, $method), $priority, $args);
    } // end addActionListener
}