<?php

class OpenGraphFrontendWordpressAdapter
{
    private $_postId;
    private $_metaOptions;
    private $_engine;
    
    private $_metaList = array(
        'type',
        'title',
        'description',
        'url',
        'image'
    );
    
    public function __construct(IFestiEngine $engine)
    {
        $this->_engine = $engine;
        $this->_postId = $this->_getPostId();
        $this->_metaOptions = $this->_getMetaFromPostOptions();
    } // __construct
    
    public function getMetaContainer()
    {        
        if (!$this->_postId || !$this->_metaOptions) {
            return false;
        }
        
        $metaList = $this->_getMetaList();
        
        if (!$metaList) {
            return false;
        }
        
        $vars = array(
            'metaList' => $metaList
        );

        return $this->_engine->fetch('open_graph/meta_block.phtml', $vars);
    } // end getMetaContainer
    
    private function _getMetaList()
    {
        $metaList = array();

        foreach ($this->_metaList as $ident) {
            $value = $this->_getMetaValue($ident);
            
            if (!$value) {
                continue;
            }
            
            $metaList[$ident] = $value;
        }
        
        return $metaList;
    } // end _getMetaList
    
    private function _getMetaValue($ident)
    {
        if ($this->_hasItemInMetaPostOptions($ident)) {
            return $this->_metaOptions[$ident];
        }

        $methodName = '_getMeta'.ucfirst($ident);

        $method = array($this, $methodName);
        
        if (!is_callable($method)) {
            throw new Exception("Undefined method name: ".$methodName);
        }

        $value = call_user_func_array($method, array());
        
        return $value;
    } // end _getMetaValue
    
    private function _hasItemInMetaPostOptions($ident)
    {
        return array_key_exists($ident, $this->_metaOptions)
               && !empty($this->_metaOptions[$ident]);
    } // end _hasItemInMetaPostOptions
    
    private function _getMetaFromPostOptions()
    {
        if (!$this->_postId) {
            return false;
        }
        
        $value = get_post_meta($this->_postId, 'festiSocialOpenGraph', true);

        if (!$value) {
            return false;
        }
        
        if (is_array($value)) {
            return $value;
        }
        
        $value = json_decode($value, true);
        
        return $value;
    } // end _getMetaFromPostOptions
    
    private function _getPostId()
    {
        return get_the_ID();
    } // end _getPostId

    private function _getMetaUrl()
    {
        $serverUrl = $_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
        $siteUrl = get_site_url();
        $url = $this->_getUrlProtocolType($siteUrl).$serverUrl;
        return $url;
    } // end _getMetaUrl
    
    private function _getUrlProtocolType($url)
    {
        $type = substr($url, 0, 5);
        $type = str_replace(':', '', $type);
        
        return $type.'://';
    } // end _getUrlProtocolType
    
    private function _getMetaType()
    {
        return 'website';
    } // end _getMetaTitle
    
    private function _getMetaTitle()
    {
        return false;
    } // end _getMetaTitle
    
    private function _getMetaDescription()
    {
        return false;
    } // end _getMetaDescription
    
    private function _getMetaImage()
    {
        return false;
    } // end _getMetaImage
}