<?php
$filePath = 'paypal/sdk-core-php-master/vendor/autoload.php';
require_once dirname(__FILE__).'/../../libs/'.$filePath;

class PaypalSocialComponent extends AbstractSocialComponent
{
    private $_paypal;
    private $_accessToken;
    public  $emailDomain = 'paypal.com';
    private $_config = array();
	private $_userData;
    
    public function onInit()
    {
    	$this->_paypal = new PPApiContext(array('mode' => 'live'));
        $this->config['appId'] = $this->getOption('appId');
        $this->config['appSecret'] = $this->getOption('appSecret');
        $scope = $this->getOption('scope');
        $this->config['scope'] = explode(" ", $scope);
        $this->config['appRedirectUrl'] = $this->getOption('appRedirectUrl');
		
		if ($this->_isLoginAction()) {
			$this->setUserDataInfo();
		}
    } // end onInit
    
    protected function setUserDataInfo()
	{
		$this->_userData = $this->getUserInfo($_GET['token']);
	} // end setUserData
	
	public function getUserDataInfo()
	{
		return $this->_userData;
	} // end getUserDataInfo
    
    private function _isLoginAction()
	{
		return isset($_GET['token'])
			   && !empty($_GET['token'])
			   && isset($_GET['social']['action'])
			   && $_GET['social']['action'] == 'login';
	} // edn _isLoginAction
    
    public function getAuthUrl()
    {        
        $openIdUrl = PPOpenIdSession::getAuthorizationUrl(
            $this->config['appRedirectUrl'],
            $this->config['scope'],
            $this->config['appId'],
            $this->_paypal
        ); 
        
        return $openIdUrl;
    } // end getAuthUrl
    
    public function getAccessToken()
    {
        $params = array(
            'client_id' => $this->config['appId'],
            'client_secret' => $this->config['appSecret'],
            'code' => $_GET['code']
        );
        
        $response = PPOpenIdTokeninfo::createFromAuthorizationCode(
            $params,
            $this->_paypal
        );
        
        $token = $response->__get('access_token');
        
        return $token;
    } // end getAuthUrl
    
    private function _hasAccessTokenInResponse($response)
    {
        return array_key_exists('access_token', $response);
    } // end _hasAccessTokenInResponse
    
    public function getUserInfo($token)
    {
        $params = array(
            'access_token' => $token
        );
        
        $user = PPOpenIdUserinfo::getUserinfo(
            $params,
            $this->_paypal
        );
        
        $userData = $user->toArray();
        
        return $userData;
    } // end getUserInfo

    public function getName()
    {
        return 'paypal';
    } // end getName
    
    public function getUserData($token)
    {
        $user = $this->getUserInfo($token);

        if (!$user) {
            $message = "The user is not logged in the".$this->getName();
            $message .= " social network.";
            throw new Exception($message);
        }

        $data = array(
            'id' => $user['email']
        );

        if (isset($user['email'])) {
             $data['email'] = $user['email'];
        }

        return $data;
    } // end getUserData
    
	protected function getAuthPageWithQuery($query)
	{
		$token = $this->getAccessToken();
		
		$url = $this->getAuthPageUrl();
		
		$args = array(
            'social' => $query['social'],
            'token' => $token
        );

        $seperator = preg_match("#\?#Umis", $url) ? '&' : '?';
        $url .= $seperator.http_build_query($args);

        return $url;
	} // end getAuthPageWithQuery
}