<?php

class FestiTwitterApp
{
    protected $consumerKey;
    protected $consumerSecret;
    protected $requestTokenUrl;
    protected $authorizeUrl;
    protected $accessTokenUrl;
    protected $accountDataUrl;
    protected $urlSeparator = '&';
	protected $callbackUrl;

    public function __construct($consumerKey, $consumerSecret, $callbackUrl)
    {
        $this->consumerKey = $consumerKey;
        $this->consumerSecret = $consumerSecret;
		$this->callbackUrl = $callbackUrl;
        
        $twitterDomain = 'https://api.twitter.com/';
        
        $this->requestTokenUrl = $twitterDomain.'oauth/request_token';
        $this->authorizeUrl = $twitterDomain.'oauth/authorize';
        $this->accessTokenUrl = $twitterDomain.'oauth/access_token';
        $this->accountDataUrl = $twitterDomain.'1.1/users/show.json';

        define('CONSUMER_KEY', $consumerKey);
        define('CONSUMER_SECRET', $consumerSecret);
        
        define('REQUEST_TOKEN_URL', 'https://api.twitter.com/oauth/request_token');
        define('AUTHORIZE_URL', 'https://api.twitter.com/oauth/authorize');

        define('ACCESS_TOKEN_URL', 'https://api.twitter.com/oauth/access_token');

        define('ACCOUNT_DATA_URL', 'https://api.twitter.com/1.1/users/show.json');

        define(
        	'CALLBACK_URL',
        	'http://'.$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"]
		);
        
        define('URL_SEPARATOR', '&');
    } // end __construct
    
    public function getToken()
    {
        $urlSeparator = $this->urlSeparator;      
        $oauthNonce = md5(uniqid(rand(), true));
        $oauthTimestamp = $this->getTwitterTimeStamp();
        
        $oauthToken = $_GET['oauth_token'];
        $oauthVerifier = $_GET['oauth_verifier'];
        
        $params = array(
            'oauth_verifier='.$oauthVerifier.$this->urlSeparator,
            'oauth_token='.$oauthToken.$this->urlSeparator,
            'oauth_consumer_key='.$this->consumerKey.$this->urlSeparator,
            'oauth_nonce='.$oauthNonce.$this->urlSeparator,
            'oauth_signature_method=HMAC-SHA1'.$this->urlSeparator,
            'oauth_timestamp='.$oauthTimestamp.$this->urlSeparator,
            'oauth_version=1.0'
        );
      
        $oauthSignature = $this->getOauthSignature(
            $params,
            $this->accessTokenUrl
        );
        
        $params = array(
            $urlSeparator.'oauth_consumer_key='.$this->consumerKey,
            'oauth_verifier='.$oauthVerifier.$this->urlSeparator,
            'oauth_nonce='.$oauthNonce,
            'oauth_signature='.urlencode($oauthSignature),
            'oauth_signature_method=HMAC-SHA1',
            'oauth_token='.$oauthToken.$this->urlSeparator,
            'oauth_timestamp='.$oauthTimestamp,
            'oauth_version=1.0'
        );

        $urlArgs = implode('&', $params);
        $url = $this->accessTokenUrl.'?'.$urlArgs;
      
        $response = $this->fileGetContentsCurl($url);
        parse_str($response, $response);
        
        return $response;
    } // end getToken
    
    public function getUserData($oauthToken, $oauthTokenSecret, $screenName)
    {
        $urlSeparator = $this->urlSeparator;      
        $oauthNonce = md5(uniqid(rand(), true));
        $oauthTimestamp = $this->getTwitterTimeStamp();
        
        
        
        $oauth_nonce = $oauthNonce;
        $oauth_timestamp =  $oauthTimestamp;

        $params = array(
            'oauth_consumer_key='.$this->consumerKey.$urlSeparator,
            'oauth_nonce='.$oauthNonce.$this->urlSeparator,
            'oauth_signature_method=HMAC-SHA1'.$this->urlSeparator,
            'oauth_timestamp='.$oauthTimestamp.$this->urlSeparator,
            'oauth_token=' . $oauthToken.$this->urlSeparator,
            'oauth_version=1.0'.$this->urlSeparator,
            'screen_name='.$screenName
        );
        
        $oauthSignature = $this->getOauthSignature(
            $params,
            $this->accountDataUrl,
            array($oauthTokenSecret)
        );
        
        
        
        $params = array(
            'oauth_consumer_key='.$this->consumerKey,
            'oauth_nonce='.$oauthNonce,
            'oauth_signature='.urlencode($oauthSignature),
            'oauth_signature_method=HMAC-SHA1',
            'oauth_timestamp='.$oauthTimestamp,
            'oauth_token='.urlencode($oauthToken),
            'oauth_version=1.0',
            'screen_name='.$screenName
        );
        
        $urlArgs = implode('&', $params);
        $url = $this->accountDataUrl.'?'.$urlArgs;
        
        $response =  $this->fileGetContentsCurl($url);

        $userData = json_decode($response);

        return $userData;
    } // end getUserData
    
    protected function getOauthSignature($params = array(), $url, $args = array())
    {
        $urlSeparator = $this->urlSeparator;
        
        $url = urlencode($url);
        
        $oauthBaseText = implode('', array_map('urlencode', $params));
        
        $oauthBaseText = 'GET'.$urlSeparator.$url.$urlSeparator.$oauthBaseText;

        $key = $this->consumerSecret.$urlSeparator;
        $key .= implode($urlSeparator, $args);;
        
        $signature = hash_hmac('sha1', $oauthBaseText, $key, true);
        $signature = base64_encode($signature);
        
        return $signature;
    } // end getOauthSignature
        
    public function getAuthUrl()
    {      
        $urlSeparator = $this->urlSeparator;      
        $oauthNonce = md5(uniqid(rand(), true));
        $oauthTimestamp = $this->getTwitterTimeStamp();

        $callbackUrl = urlencode($this->getCallbackUrl());

        $params = array(
            'oauth_callback=' . $callbackUrl.$this->urlSeparator,
            'oauth_consumer_key='.$this->consumerKey.$this->urlSeparator,
            'oauth_nonce='.$oauthNonce.$this->urlSeparator,
            'oauth_signature_method=HMAC-SHA1'.$this->urlSeparator,
            'oauth_timestamp='.$oauthTimestamp.$this->urlSeparator,
            'oauth_version=1.0'
        );
		

      
        $oauthSignature = $this->getOauthSignature(
            $params,
            $this->requestTokenUrl
        );
        
        $params = array(
            $urlSeparator.'oauth_consumer_key='.$this->consumerKey,
            'oauth_nonce='.$oauthNonce,
            'oauth_signature='.urlencode($oauthSignature),
            'oauth_signature_method=HMAC-SHA1',
            'oauth_timestamp='.$oauthTimestamp,
            'oauth_version=1.0'
        );

        $urlArgs = implode('&', $params);
        $url = $this->requestTokenUrl.'?oauth_callback='.$callbackUrl.$urlArgs;
      
        $response = $this->fileGetContentsCurl($url);

        parse_str($response, $response);

        $oauthToken = $response['oauth_token'];
        $oauthTokenSecret = $response['oauth_token_secret'];
        
        $link = $this->authorizeUrl.'?oauth_token='.$oauthToken;

        return $link;
       
    } // end getAuthUrl
    
    protected function getCallbackUrl()
    {		
        return $this->callbackUrl;
    } // end getCallbackUrl
    
    function fileGetContentsCurl($url, $returnInfo = false) {
    	$ch = curl_init();
		
        curl_setopt($ch, CURLOPT_AUTOREFERER, true);
        curl_setopt($ch, CURLINFO_FILETIME, true);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_FILETIME, true);
		if ($returnInfo) {
			curl_setopt($ch, CURLOPT_HEADER, 1);
		}
        
        $data = curl_exec($ch);
        $info = curl_getinfo($ch); 
		
		if ($returnInfo) {
			$pattern = '/date:(.*GMT)/isU';

	        preg_match($pattern, $data, $matches);

	        if ($matches) {
	        	$timestamp = strtotime($matches[1]);

	        	return $timestamp;
	        }
		}

        curl_close($ch);
        
        if (!$returnInfo) {
            return $data;
        } else {
            return $info;
        }
    } // end fileGetContentsCurl
    
    
    public function getTwitterTimeStamp()
    {
        $info = $this->fileGetContentsCurl($this->requestTokenUrl, true);

		if (is_integer($info)) {
			return $info;
		}
		
        return $info['filetime'];
    } // end getTwitterTimeStamp
}
