<?php
/**
 * Plugin Name: WooCommerce Social Plugin
 * Plugin URI: http://festi.io/app/festi-social-login-plugin/
 * Description:   WooCommerce Social Plugin helps you integrate social network options on you web-store.
 * Version: 1.08.01
 * Author: Festi 
 * Author URI: http://festi.io/
 */

if (!class_exists('FestiPlugin')) {
    require_once dirname(__FILE__).'/common/FestiPlugin.php';
}

if (!class_exists('WoocommerceFestiPlugin')) {
    require_once dirname(__FILE__).'/common/WoocommerceFestiPlugin.php';
}

require_once dirname(__FILE__).'/common/social/WordpressSocialAdapter.php';

class WooSocialFestiPlugin extends WoocommerceFestiPlugin
{
    public $_languageDomain = 'festi_social_';
    protected $_optionsPrefix = 'festi_social_';
    
    public $version = '1.08.01';

    public $_socialNetworks = array(
        'facebook',
        'vk',
        'gplus',
        'twitter',
        'paypal'
    );
	
	protected function onInit()
	{
		parent::onInit();
		
		$this->addActionListener('init', 'onRewriteRulesAction');
	} // end onInit
	
    public function onRewriteRulesAction()
    {
    	$this->_onAppendRewriteTags();
    } // end onRewriteRulesAction
    
    private function _onAppendRewriteTags()
	{
		add_rewrite_tag('%social%', '([^&]+)');
	} // end _onAppendRewriteTags
    
    protected function getPluginFolderName()
    {
        return 'woocommerce-social-plugin';
    } // end getPluginFolderName
    
    protected function onFrontendInit()
    {
        $fileName = 'WooSocialFrontendFestiPlugin.php';
        require_once $this->_pluginPath.'/'.$fileName;
        $frontend = new WooSocialFrontendFestiPlugin(__FILE__);
        return $frontend;
    } // end onFrontendInit
    
    protected function onBackendInit()
    {
        $fileName = 'WooSocialBackendFestiPlugin.php';
        require_once $this->_pluginPath.'/'.$fileName;
        $backend = new WooSocialBackendFestiPlugin(__FILE__);
        return $backend;
    } // end onBackendInit
    
    public function getAuthPageUrl()
	{
		if (!$this->isExistsAuthPage()) {
			$this->doCreateAuthPage();
		}
		
		$value = $this->getOptions('auth_page');
		
		$postId = $value[0];

		return get_permalink($postId);
	} // end getAuthPageUrl
    
    protected function doCreateAuthPage()
	{
		$post = array(
			'post_title'   => 'WooSocial',
			'post_content' => '',
			'post_status'  => 'publish',
			'post_type'    => 'page'
		);
	
		$pageId = wp_insert_post($post);
		
		$this->updateOptions('auth_page', array($pageId));
	} // end doCreateAuthPage
	
    protected function isExistsAuthPage()
	{
		$post = $this->getOptions('auth_page');

		if (!$post) {
			return false;
		}
		
		$postId = $post[0];

		return get_post($postId);
	} // end isExistsAuthPage
}

$GLOBALS['wooSocialFestiPlugin'] = new WooSocialFestiPlugin(__FILE__);